// PacketLogPlugIn.cpp: implementation of the CPacketLogPlugIn class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Packet Log.h"
#include "PacketLogPlugIn.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPacketLogPlugIn::CPacketLogPlugIn()
	: CPlugIn(CLSID_PacketLogPlugIn, "Packet Log PlugIn", PLUGINVERSION(1, 0, 30), "Jarrett"), m_szFileName("PACKET.LOG"), m_szFontName("Courier New")
{
	m_bLogToFile = FALSE;
	m_bEnableLogging = FALSE;
	m_nFontSize = 10;
	m_nUpdateInterval = 5000;

	m_hpibar = NULL;
	m_hpioptions = NULL;
	m_hpiview = NULL;
	m_hpimenu = NULL;
	m_hpihelp = NULL;

}

CPacketLogPlugIn::~CPacketLogPlugIn()
{

}
void CPacketLogPlugIn::AddText(char* pszHeader, BYTE* pbBuffer, UINT nNumberOfBytesToAdd) 
{

	if (m_bEnableLogging) {

		CString szText;
		CString szTemp;

		szText.Format("[%s:%d] ", pszHeader, nNumberOfBytesToAdd);

		for (UINT i = 0; i < nNumberOfBytesToAdd; i++) {

			szTemp.Format("%02x ", pbBuffer[i]);

			szText += szTemp;

		}

		m_pview->AddText(szText);

		m_cs.Lock();

		if (m_bLogToFile) {

			CStdioFile file;

			if (file.Open(m_szFileName, CFile::modeCreate | CFile::modeNoTruncate | CFile::modeWrite | CFile::shareDenyWrite)) {

				file.SeekToEnd();

				file.WriteString(szText);
				file.WriteString("\n");
				file.Close();

			}

		}

		m_cs.Unlock();
	}



}

BOOL CPacketLogPlugIn::OnSend(BYTE* pbBuffer, UINT nNumberOfBytesSent, BOOL bFake, BOOL bSelfCreated)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	if (bFake) {
		AddText("FAKESEND", pbBuffer, nNumberOfBytesSent);
	} else {
		AddText("SEND", pbBuffer, nNumberOfBytesSent);
	}
	
	return FALSE;
}

BOOL CPacketLogPlugIn::OnReceive(BYTE* pbBuffer, UINT nNumberOfBytesReceived, BOOL bFake, BOOL bSelfCreated)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	if (bFake) {
		AddText("FAKERECV", pbBuffer, nNumberOfBytesReceived);
	} else {
		AddText("RECV", pbBuffer, nNumberOfBytesReceived);
	}

	return FALSE;
}

BOOL CPacketLogPlugIn::InitInstance(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_pview = new CPacketLogView(this);
	m_poptions = new CPacketLogOptionsDlg(this);

	if (!m_pview->Create())
		return FALSE;

	m_hpibar = AddBar(m_pview->m_wndToolBar.GetSafeHwnd());
	m_hpiview = AddView(m_pview->GetSafeHwnd());
	m_hpimenu = AddMenu("&Log", m_pview->m_hmenu, m_pview->GetSafeHwnd());
	m_hpioptions = AddOptions((PROPSHEETPAGE*)&m_poptions->m_psp);
	m_hpihelp = AddHelp("Packet Log PlugIn", m_pview->GetSafeHwnd(), ID_PACKETLOG_HELP);

	return TRUE;
}

void CPacketLogPlugIn::ExitInstance(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	if (m_hpibar)
	{
		RemoveBar(m_hpibar);
		m_hpibar = NULL;
	}
	if (m_hpiview)
	{
		RemoveView(m_hpiview);
		m_hpiview = NULL;
	}
	if (m_hpimenu)
	{
		RemoveMenu(m_hpimenu);
		m_hpimenu = NULL;
	}
	if (m_hpioptions)
	{
		RemoveOptions(m_hpioptions);
		m_hpioptions = NULL;
	}
	if (m_hpihelp)
	{
		RemoveHelp(m_hpihelp);
		m_hpihelp = NULL;
	}

	delete m_pview;
	delete m_poptions;
}

void CPacketLogPlugIn::ReadSettings(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_cs.Lock();

	ReadSetting("Enabled", (DWORD*)&m_bEnableLogging, TRUE);

	ReadSetting("FontName", (PSTR)m_szFontName.GetBuffer(1024), 1024);
	m_szFontName.ReleaseBuffer();

	ReadSetting("FontSize", (DWORD*)&m_nFontSize);

	ReadSetting("UpdateInterval", (DWORD*)&m_nUpdateInterval);

	ReadSetting("LogToFile", (DWORD*)&m_bLogToFile);
	ReadSetting("FileName", (PSTR)m_szFileName.GetBuffer(1024), 1024);
	m_szFileName.ReleaseBuffer();

	m_cs.Unlock();

}

void CPacketLogPlugIn::WriteSettings(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_cs.Lock();

	WriteSetting("Enabled", (DWORD)m_bEnableLogging, TRUE);

	WriteSetting("FontName", (PCSTR)m_szFontName);
	WriteSetting("FontSize", (DWORD)m_nFontSize);

	WriteSetting("UpdateInterval", (DWORD)m_nUpdateInterval);

	WriteSetting("LogToFile", (DWORD)m_bLogToFile);
	WriteSetting("FileName", (PCSTR)m_szFileName);

	m_cs.Unlock();

}

void CPacketLogPlugIn::GetFileName(CString& szFileName)
{
	m_cs.Lock();

	szFileName = m_szFileName;

	m_cs.Unlock();
}

BOOL CPacketLogPlugIn::GetLogToFile(void)
{
	BOOL bLogToFile;

	m_cs.Lock();

	bLogToFile = m_bLogToFile;

	m_cs.Unlock();

	return bLogToFile;
}

BOOL CPacketLogPlugIn::GetEnableLogging(void)
{
	BOOL bEnableLogging;

	m_cs.Lock();

	bEnableLogging = m_bEnableLogging;

	m_cs.Unlock();

	return bEnableLogging;
}

void CPacketLogPlugIn::GetFontName(CString& szFontName)
{
	m_cs.Lock();

	szFontName = m_szFontName;

	m_cs.Unlock();
}

UINT CPacketLogPlugIn::GetFontSize(void)
{
	UINT nFontSize;

	m_cs.Lock();

	nFontSize = m_nFontSize;

	m_cs.Unlock();

	return nFontSize;
}

UINT CPacketLogPlugIn::GetUpdateInterval(void)
{
	UINT nUpdateInterval;

	m_cs.Lock();

	nUpdateInterval = m_nUpdateInterval;

	m_cs.Unlock();

	return nUpdateInterval;
}


void CPacketLogPlugIn::SetFileName(CString& szFileName)
{
	m_cs.Lock();

	m_szFileName = szFileName;

	m_cs.Unlock();
}

void CPacketLogPlugIn::SetLogToFile(BOOL bLogToFile)
{
	m_cs.Lock();

	m_bLogToFile = bLogToFile;

	m_cs.Unlock();
}

void CPacketLogPlugIn::SetEnableLogging(BOOL bEnableLogging)
{
	m_cs.Lock();

	m_bEnableLogging = bEnableLogging;

	m_cs.Unlock();
}

void CPacketLogPlugIn::SetFontName(CString& szFontName)
{
	m_cs.Lock();

	m_szFontName = szFontName;

	m_cs.Unlock();

	m_pview->PostMessage(PLM_FONTCHANGED);
}

void CPacketLogPlugIn::SetFontSize(UINT nFontSize)
{
	m_cs.Lock();

	m_nFontSize = nFontSize;

	m_cs.Unlock();

	m_pview->PostMessage(PLM_FONTCHANGED);
}

void CPacketLogPlugIn::SetUpdateInterval(UINT nUpdateInterval)
{
	m_cs.Lock();

	m_nUpdateInterval = nUpdateInterval;

	m_cs.Unlock();

	m_pview->PostMessage(PLM_UPDATEINTERVALCHANGED);
}

