// PacketLogView.cpp : implementation file
//

#include "stdafx.h"
#include "Packet Log.h"
#include "PacketLogPlugIn.h"
#include "PacketLogView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPacketLogView dialog


CPacketLogView::CPacketLogView(CPacketLogPlugIn* pplpi)
	: CDialog(CPacketLogView::IDD, NULL)
{
	//{{AFX_DATA_INIT(CPacketLogView)
	//}}AFX_DATA_INIT

	m_pplpi = pplpi;
}


void CPacketLogView::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPacketLogView)
	DDX_Control(pDX, IDC_PACKETLOG, m_txt);
	//}}AFX_DATA_MAP
}

BOOL CPacketLogView::Create(void) 
{
	return CDialog::Create(IDD, NULL);
}

LRESULT CPacketLogView::OnAddText(WPARAM wParam, LPARAM lParam)
{

	m_szPacketLog += (PSTR)wParam;
	m_szPacketLog += "\r\n";
	
	return 0;
}

void CPacketLogView::AddText(PCSTR pcszText) 
{
	SendMessage(PLM_ADDTEXT, (WPARAM)pcszText);
}

static UINT WM_FINDREPLACE = ::RegisterWindowMessage(FINDMSGSTRING);

BEGIN_MESSAGE_MAP(CPacketLogView, CDialog)
	//{{AFX_MSG_MAP(CPacketLogView)
	ON_WM_CTLCOLOR()
	ON_WM_SIZE()
	ON_COMMAND(ID_PACKETLOG_TOGGLE, OnPacketLogToggle)
	ON_COMMAND(ID_PACKETLOG_CLEAR, OnPacketLogClear)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_PACKETLOG_HELP, OnPacketLogHelp)
	ON_MESSAGE(PLM_ADDTEXT, OnAddText)
	ON_MESSAGE(PLM_FONTCHANGED, OnFontChanged)
	ON_MESSAGE(PLM_UPDATEINTERVALCHANGED, OnUpdateIntervalChanged)
	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, OnToolTipsNeedText)
	ON_MESSAGE(PIM_GETVERBS, OnGetVerbs)
	ON_MESSAGE(PIM_EXECUTEVERB, OnExecuteVerb)
	ON_REGISTERED_MESSAGE( WM_FINDREPLACE, OnFindReplace)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPacketLogView message handlers

HBRUSH CPacketLogView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr;
	
	switch (nCtlColor) {
	case CTLCOLOR_DLG:
		hbr = (HBRUSH)GetStockObject(NULL_BRUSH);
		break;
	default:
		hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
		break;
	}
	
	return hbr;
}

BOOL CPacketLogView::OnInitDialog() 
{
	CDialog::OnInitDialog();

	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT | TBSTYLE_TOOLTIPS) ||
		!m_wndToolBar.LoadToolBar(IDR_PACKETLOG))
		return FALSE;

	CSize size;

	size = m_wndToolBar.CalcFixedLayout(FALSE, TRUE);
	m_wndToolBar.MoveWindow(0, 0, size.cx, size.cy);

	SetIcon((HICON)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_PACKETLOG), IMAGE_ICON, 16, 16, 0), FALSE);
	SetIcon((HICON)LoadImage(AfxGetResourceHandle(), MAKEINTRESOURCE(IDI_PACKETLOG), IMAGE_ICON, 32, 32, 0), TRUE);

	m_wndToolBar.GetToolBarCtrl().PressButton(ID_PACKETLOG_TOGGLE, m_pplpi->GetEnableLogging());

	UINT nFontSize;
	CString szFontName;

	nFontSize = m_pplpi->GetFontSize();
	m_pplpi->GetFontName(szFontName);

	m_font.CreatePointFont(nFontSize * 10, szFontName);

	m_txt.SetFont(&m_font);

	m_hmenu = LoadMenu(AfxGetResourceHandle(), MAKEINTRESOURCE(IDR_PACKETLOG));

	CheckMenuItem(m_hmenu, 0, MF_BYPOSITION | (m_pplpi->GetEnableLogging() ? MF_CHECKED : MF_UNCHECKED));

	SetTimer(IDT_PACKETLOG, m_pplpi->GetUpdateInterval(), NULL);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

LRESULT CPacketLogView::OnFontChanged(WPARAM wParam, LPARAM lParam)
{

	UINT nFontSize;
	CString szFontName;

	nFontSize = m_pplpi->GetFontSize();
	m_pplpi->GetFontName(szFontName);

	m_font.DeleteObject();

	m_font.CreatePointFont(nFontSize * 10, szFontName);

	m_txt.SetFont(&m_font);

	m_txt.Invalidate();

	return 0;
}

LRESULT CPacketLogView::OnUpdateIntervalChanged(WPARAM wParam, LPARAM lParam)
{

	KillTimer(IDT_PACKETLOG);

	SetTimer(IDT_PACKETLOG, m_pplpi->GetUpdateInterval(), NULL);

	return 0;
}


void CPacketLogView::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if (::IsWindow(m_txt))
	{
		m_txt.MoveWindow(0, 0, cx, cy);
		m_txt.SendMessage(EM_SCROLLCARET);
	}
}

void CPacketLogView::OnPacketLogToggle() 
{
	m_pplpi->SetEnableLogging(!m_pplpi->GetEnableLogging());

	m_wndToolBar.GetToolBarCtrl().PressButton(ID_PACKETLOG_TOGGLE, m_pplpi->GetEnableLogging());
	CheckMenuItem(m_hmenu, 0, MF_BYPOSITION | (m_pplpi->GetEnableLogging() ? MF_CHECKED : MF_UNCHECKED));
	
}

void CPacketLogView::OnPacketLogClear() 
{
	m_txt.SetWindowText("");

	m_szPacketLog = "";
	
}

void CPacketLogView::OnTimer(UINT nIDEvent) 
{
	switch (nIDEvent) {
	case IDT_PACKETLOG:
		if (!m_szPacketLog.IsEmpty()) {
			m_txt.SetSel(-1, -1);
			m_txt.ReplaceSel((PCSTR)m_szPacketLog);
			m_txt.SendMessage(EM_SCROLLCARET);

			m_szPacketLog = "";
		}
		break;
	}
	
	CDialog::OnTimer(nIDEvent);
}

BOOL CPacketLogView::OnToolTipsNeedText(UINT id, NMHDR* pNMHDR, LRESULT* pResult)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    UINT nID =pNMHDR->idFrom;

    if (!(pTTT->uFlags & TTF_IDISHWND))
    {
		CString szToolTip;

		szToolTip.LoadString(nID);

		szToolTip = szToolTip.Mid(szToolTip.Find('\n') + 1);
		strcpy(pTTT->szText, szToolTip);

		return TRUE;

    }
    
	return FALSE;
}

LRESULT CPacketLogView::OnGetVerbs(WPARAM wParam, LPARAM lParam)
{
	return PIV_COPY | PIV_FIND;
}

LRESULT CPacketLogView::OnExecuteVerb(WPARAM wParam, LPARAM lParam)
{
	switch (wParam)
	{
	case PIV_COPY:
		m_txt.Copy();
		break;
	case PIV_FIND: 
		{

		CFindReplaceDialog* pdlg = new CFindReplaceDialog;

		pdlg->Create(TRUE, m_szFindText, "", FR_HIDEUPDOWN | FR_DOWN, this);
		
		break; 
		}
	}

	return 0;
}

LRESULT CPacketLogView::OnFindReplace(WPARAM wParam, LPARAM lParam)
{
	FINDREPLACE* pfr = (FINDREPLACE*)lParam;

	if (pfr->Flags & FR_FINDNEXT)
	{
		FINDTEXTEX fte;

		m_txt.GetSel(fte.chrg);

		fte.chrg.cpMin = fte.chrg.cpMax;
		fte.chrg.cpMax = -1;
		fte.lpstrText = pfr->lpstrFindWhat;

		int iSel = m_txt.FindText(pfr->Flags, &fte);

		if (iSel != -1)
		{
			m_txt.SetSel(fte.chrgText);

		}
		else
		{
			fte.chrg.cpMin = 0;
			fte.chrg.cpMax = -1;
			fte.lpstrText = pfr->lpstrFindWhat;

			iSel = m_txt.FindText(pfr->Flags, &fte);
			if (iSel != -1)
			{
				m_txt.SetSel(fte.chrgText);

			}
			else
			{
				MessageBox("Text not found.", NULL, MB_ICONERROR);
			}
			
		}
	
	}
	else if (pfr->Flags & FR_DIALOGTERM)
	{
		m_szFindText = pfr->lpstrFindWhat;
	}

	return 0;
}

void CPacketLogView::OnPacketLogHelp(void)
{
	MessageBox("Packet Log PlugIn help placeholder.");
}